/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#include <qlabel.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qstatusbar.h>
#include <qpixmap.h>
#include <qtooltip.h>
#include <qstylesheet.h>
#include <qregexp.h>

#include "dictOnline.h"
#include "preferences.h"

DictOnline::DictOnline( QString host, QWidget *parent, const char *name,WFlags)
	: QDialog( parent, name)
{
	QFont myFont( "Arial", 8 );
	setFont( myFont );

	/* Dictionary connector */
	dictionary = new DICTClient( this, host );

	/* Top image */
	QPixmap headerpix( qApp->applicationDirPath()+"/../images/dict.png" );
	QLabel *header = new QLabel( this );
	header->setPixmap( headerpix );	

	/* Label */
	QLabel *label = new QLabel ( "<b>Enter a word here:</b>", this );
	label->setFrameStyle( QFrame::Box | QFrame::Sunken );

	/* Entry area */
	QHBox *hbox = new QHBox( this );
	line        = new QLineEdit( hbox);
	okbutton    = new QPushButton( "Define", hbox );
	okbutton->setFixedSize(80,24);
	line->setFocus();
	hbox->setSpacing( 2 );

	/* Combo box, default dictionaries */
	combo = new QComboBox( this );
	combo->insertItem( "* (Any dictionary)" );
	combo->insertItem( "web1913 \"Webster's Revised Unabridged Dictionary (1913)\"" );
	combo->insertItem( "wn \"WordNet (r) 1.7\"" );
	combo->setCurrentItem( 0 );

	/* Text view */
	view = new QTextView( this );

	/* Status bar */
	statusbar = new QStatusBar( this );
	QString servername = QString( " Server " ) + host + " ";
	QLabel *database = new QLabel( servername, statusbar );
	database->setFont( QFont( "Arial", 6 ) );
	move = new DownloadBar( qApp->applicationDirPath()+"/../images/download.png", 24, statusbar );
	statusbar->addWidget( database, 0, true );
	statusbar->addWidget( move, 0 , true );
	statusbar->setFixedHeight( 18 );
	statusbar->message( "Not connected" );
	statusbar->setFont( myFont );

	/* Vertical area */
	QVBoxLayout *vbox = new QVBoxLayout( this );
	vbox->setResizeMode( QLayout::FreeResize );
	vbox->add( header    );
	vbox->add( label     );
	vbox->add( hbox      );
	vbox->add( combo     );
	vbox->add( view      );
	vbox->add( statusbar );
	vbox->setSpacing( 2 );

	/* Events and signals */
	connect( line, SIGNAL(returnPressed()), this, SLOT(search()) );
	connect( okbutton, SIGNAL(clicked()), this, SLOT(search()) );

	/* Default text */
    view->setText( 
		"<center><br/><br/><br/><br/><b>Dragon Flame</b><br/>"
		" 2002-2003 Benjamin Babut & Didier Willis.<br/>"
		"- <i>Dragon Flame</i> is free software -<br/>"
		"</center>" );
		
	/* Background for the text area */
	QBrush paper;
	paper.setPixmap( QPixmap( qApp->applicationDirPath()+"/../images/paper.png" ) );
    view->setPaper( paper );
	QString unifontname = DFConfig::instance()->getString( "fonts", "unicode", "Lucida Sans Unicode" );
	view->setFont( QFont( unifontname, 8 ) );

	/* Tips */
    QToolTip::add( okbutton, "Search a definition" );
	QToolTip::add( combo,    "Select a database" );
	QToolTip::add( line,     "Enter a word to search" );

	/* Other settings */
	setFixedSize( 430, 500 );

	/* Initial connection to retrieve databases */
	if (DFConfig::instance()->getBool( "dictOnline", "enable", false ))
	{
		line->setEnabled(     false );
		okbutton->setEnabled( false );
		combo->setEnabled(    false );
		statusbar->message( "Connecting..." );
		move->start();
		dictionary->define( "" );
	}
}


void DictOnline::customEvent( QCustomEvent *e )
{
	if ( e->type() == static_cast<QEvent::Type>(DICTClientEvent::Definition))
	{
		/* 
		 * Search result...
		 */
		DICTClientEvent *dictQuery = (DICTClientEvent *)e;
		QString result = QStyleSheet::convertFromPlainText( dictQuery->result() );

		result.replace( QRegExp( "--@" ), "<hr/><b>" );
		result.replace( QRegExp( "@--" ), "</b>" );
		result.replace( QRegExp( "{" ), "<i>" );
		result.replace( QRegExp( "}" ), "</i>" );

		view->setText( result );
		line->setEnabled(     true );
		okbutton->setEnabled( true );
		combo->setEnabled(    true );
		move->stop();
		statusbar->message( "Ready." );
		line->setFocus();
	}	
	if ( e->type() == static_cast<QEvent::Type>(DICTClientEvent::Database)) 
	{
		/*
		 * Update the available dictionaries 
		 */
		DICTClientEvent *dictQuery = (DICTClientEvent *)e;

		combo->clear();
		combo->insertItem( "* (Any dictionary)" );
		combo->insertStrList( dictQuery->database() );
		combo->setCurrentItem( 0 );	
	}	
}

void DictOnline::search()
{
	QString word = line->text();
	QString dict = combo->currentText();

	/* The name of the dictionary is the first item */
	int pos = dict.find( ' ' );
	dict = dict.left( pos );

	if (word != "")
	{
		line->setEnabled(     false );
		okbutton->setEnabled( false );
		combo->setEnabled(    false );
		statusbar->message( "Busy..." );
		move->start();

		dictionary->define( word, dict );
	}
}

/* --- THE NIFTY METHOD FOR OTHER CLASSES --- */

void DictOnline::define( QString word )
{
	if (!dictionary->running())
	{
		line->setText( word );
		this->search();
	}
	this->show();
}

/* EOF */
