/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * ---------------
 *
 * The 'Preferences/Configuration' code is based on a Qt module written
 * by David Johnson, 'Preferences version 2', copyright (c) 2000, David 
 * Johnson, david@usermode.org
 * -- Heavily modified for Dragon Flame.
 */
#include <qdom.h>
#include <qfile.h>
#include <qtextstream.h>

#include "preferences.h"

Preferences::Preferences(const QString& filename,
                         const QString& format,
                         const QString& version)
    : dirty_(false),
      file_(filename),
      format_(format),
      version_(version),
      filestate_(false),
      formatstate_(false),
      groups_()
{
    readData();
    dirty_ = false;
}

Preferences::~Preferences()
{
	// Save preferences on exit if modified.
    if (dirty_) 
	{
		writeData();
	}
}

bool Preferences::getBool( const QString& group, const QString& key, bool def )
{
    buffer_ = getString( group, key, def ? "true" : "false" );
    if (buffer_.isEmpty()) return def;
    if (buffer_.contains( "true" ))
        return true;
    else
        return false;
}

void Preferences::setBool( const QString& group, const QString& key, bool value )
{
    groups_[group][key] = value ? "true" : "false";
    dirty_ = true;
}

long Preferences::getNumber( const QString& group, const QString& key, long def )
{
    buffer_ = getString( group, key, QString::number(def));
	if (buffer_.isEmpty()) return def;

    bool ok;
    long num = buffer_.toLong(&ok);
    if (ok) return num;
    else return def;
}

void Preferences::setNumber( const QString& group, const QString& key, long value )
{
    buffer_.setNum(value);

    groups_[group][key] = buffer_;
    dirty_ = true;
}

QString Preferences::getString( const QString& group, const QString& key, const QString& def )
{
    buffer_ = "";
    if (groups_.contains( group ))
	{
        if (groups_[group].contains(key)) 
		{
            buffer_ = groups_[group][key];
        }
    }
	if (buffer_.isEmpty()) return def;
    return buffer_;
}

void Preferences::setString( const QString& group, const QString& key, const QString& value )
{
    groups_[group][key] = value;
    dirty_ = true;
}
		
void Preferences::removeKey( const QString& group, const QString& key )
{
    groups_[group].remove(key);
}

//////////////////////////////////////////////////////////////////////////////
// removeGroup()
// -------------
// Remove a group from the preferences, and all its options

void Preferences::removeGroup( const QString& group )
{
    groups_.remove( group );
}

void Preferences::flush()
{
    if (dirty_) {
        writeData();
        dirty_ = false;
    }
}

// Serialization                                                            //

void Preferences::readData()
{
    // open file
    QFile* datafile = new QFile(file_);
    if (!datafile->open(IO_ReadOnly)) {
        // error opening file
        qWarning("Error: cannot open preferences file " + file_);
        datafile->close();
        delete (datafile);
        filestate_ = false;
        return;
    }
    filestate_ = true;

    // open dom document
    QDomDocument doc("preferences");
    if (!doc.setContent(datafile)) {
        qWarning("Error: " + file_ + " is not a proper preferences file");
        datafile->close();
        delete (datafile);
        formatstate_ = false;
        return;
    }
    datafile->close();
    delete (datafile);

    // check the doc type and stuff
    if (doc.doctype().name() != "preferences") {
        // wrong file type
        qWarning("Error: " +file_ + " is not a valid preferences file");
        formatstate_ = false;
        return;
    }
    QDomElement root = doc.documentElement();
    if (root.attribute("application") != format_) {
        // right file type, wrong application
        qWarning("Error: " + file_ + " is not a preferences file for " + format_);
        formatstate_ = false;
        return;
    }
    // We don't care about application version...

    // get list of groups
    QDomNodeList nodes = root.elementsByTagName("group");

    // iterate through the groups
    QDomNodeList options;
    for (unsigned n=0; n<nodes.count(); ++n) {
        if (nodes.item(n).isElement()) {
            processGroup(nodes.item(n).toElement());
        }
    }
    formatstate_ = true;
}

void Preferences::processGroup(QDomElement group)
{
    QDomElement elem;
    QDomNodeList options;
    QString currentgroup = group.attribute("name", "Default");
    options = group.elementsByTagName("option");
    for (unsigned n=0; n<options.count(); ++n) {
        if (options.item(n).isElement()) {
            elem = options.item(n).toElement();
            setString( currentgroup, elem.attribute("key"), elem.attribute("value"));
        }
    }
}

void Preferences::writeData()
{
    QDomDocument doc("preferences");

    // create the root element
    QDomElement root = doc.createElement(doc.doctype().name());
    root.setAttribute("version", version_);
    root.setAttribute("application", format_);

    // now do our options group by group
    QMap<QString, PrefMap>::Iterator git;
    PrefMap::Iterator pit;
    QDomElement group, option;
    for (git = groups_.begin(); git != groups_.end(); ++git) {
        // create a group element
        group = doc.createElement("group");
        group.setAttribute("name", git.key());
        // add in options
        for (pit = (*git).begin(); pit != (*git).end(); ++pit) {
            option = doc.createElement("option");
            option.setAttribute("key", pit.key());
            option.setAttribute("value", pit.data());
            group.appendChild(option);
        }
        root.appendChild(group);
    }
    doc.appendChild(root);

    // open file
    QFile* datafile = new QFile(file_);
    if (!datafile->open(IO_WriteOnly)) {
        // error opening file
        qWarning("Error: Cannot open preferences file " + file_);
        datafile->close();
        delete (datafile);
        filestate_ = false;
        return;
    }
    filestate_ = true;

    // write it out
    QTextStream textstream(datafile);
    doc.save(textstream, 0);
    datafile->close();
    delete (datafile);
    formatstate_ = true;
}

DFConfig *DFConfig::singleton_ = 0;
QMutex    DFConfig::mutex;

/* EOF */
