/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * ---------------
 *
 * The 'Preferences/Configuration' code is based on a Qt module written
 * by David Johnson, 'Preferences version 2', copyright (c) 2000, David 
 * Johnson, david@usermode.org
 * -- Heavily modified for Dragon Flame.
 */

#ifndef PREFERENCES_H
#define PREFERENCES_H

#include <qapplication.h>
#include <qstring.h>
#include <qmap.h>
#include <qthread.h>

class QDomElement;

class Preferences {
public:
    Preferences( const QString& filename, const QString& format, const QString& version );
    virtual ~Preferences();

    // preference file information
    const QString& file();
    const QString& format();
    const QString& version();
    // did file open successfully?
    bool fileState();
    // is this a proper preferences file for format?
    bool formatState();

    // boolean data storage
	bool getBool( const QString& group, const QString& key, bool def = false );
    void setBool( const QString& group, const QString& key, bool value);

    // integer data storage
    long getNumber( const QString& group, const QString& key, long def = 0 );
    void setNumber( const QString& group, const QString& key, long value );

    // string data storage
	QString getString( const QString& group, const QString& key, const QString& def = "NULL");
    void setString( const QString& group, const QString& key, const QString& value);
	
    // remove a key/value from the preferences
    void removeKey( const QString& group, const QString& key );
    // remove the current group from the preferences
    void removeGroup( const QString& group );

    // flush the preferences out to file
    void flush();

protected:
    // serialization
    void readData();
    void writeData();
    void processGroup(QDomElement group);

private:
    bool dirty_;
    QString file_;
    QString format_;
    QString version_;
    QString buffer_;
    bool filestate_;
    bool formatstate_;

    typedef QMap<QString, QString> PrefMap;
    QMap<QString, PrefMap> groups_;
};

inline const QString& Preferences::file() { return file_; };
inline const QString& Preferences::format() { return format_; }
inline bool Preferences::fileState() { return filestate_; }
inline bool Preferences::formatState() { return formatstate_; }

class DFConfig : public Preferences
{
  public:
	// Thread-safe Singleton pattern
	static DFConfig *instance()
	{
		// Double-checked Locking pattern
		if (!singleton_)
		{
			mutex.lock();
			if (!singleton_)
			{
				singleton_ = new DFConfig();
			}
			mutex.unlock();
		}
		return( singleton_ );
	}
  private:
	DFConfig()
		: Preferences( qApp->applicationDirPath()+"/../config/df.xml", "Dragon Flame", "2.0" )
	{
	}
	static DFConfig *singleton_;
	static QMutex mutex;
};

#endif // PREFERENCES
